@@ -1,11 +1,12 @@
+import ipaddress
 import re
-from typing import Dict, Any
+from typing import Dict, Any, Union
 
 from .parse import BaseParser, IPBaseKeys
+from .servers import IPv4Allocations
 
 
 class RIRParser(BaseParser):
-
     date_keys = (
         IPBaseKeys.ORG_REG_DATE,
         IPBaseKeys.ORG_UPDATED,
@@ -57,12 +58,17 @@ class RIRParser(BaseParser):
 
 
 class NumberParser:
-    def __init__(self, rir_server: str):
-        self.parser_output: Dict[IPBaseKeys, Any] = {}
-        self._parser = self._init_parser(rir_server)
+    def __init__(self):
+        self.servers = IPv4Allocations()
 
-    def parse(self, blob: str) -> None:
-        self.parser_output = self._parser.parse(blob)
+    def parse(
+        self,
+        blob: str,
+        ip: Union[ipaddress.IPv4Address],
+    ) -> dict[IPBaseKeys, Any]:
+        _, server = self.servers.get_servers(ip)
+        parser = self._init_parser(server)
+        return parser.parse(blob)
 
     @staticmethod
     def _init_parser(rir_server: str) -> RIRParser:
@@ -81,12 +87,10 @@ class NumberParser:
 
 
 class ARINParser(RIRParser):  # default
-    def __init__(self):
-        ...
+    def __init__(self): ...
 
 
 class AFRINICParser(RIRParser):
-
     _afrinic = {
         IPBaseKeys.NET_RANGE: r"inetnum: *(.+)",
         IPBaseKeys.NET_NAME: r"netname: *(.+)",
@@ -150,7 +154,6 @@ class AFRINICParser(RIRParser):
 
 
 class APNICParser(RIRParser):
-
     _apnic = {
         IPBaseKeys.NET_RANGE: r"inetnum: *(.+)",
         IPBaseKeys.NET_NAME: r"netname: *(.+)",
@@ -221,7 +224,6 @@ class APNICParser(RIRParser):
 
 
 class LACNICParser(RIRParser):
-
     _lacnic = {
         IPBaseKeys.NET_RANGE: r"inetnum: *(.+)",
         IPBaseKeys.CIDR: r"inetrev: *(.+)",
@@ -291,7 +293,6 @@ class LACNICParser(RIRParser):
 
 
 class RIPEParser(RIRParser):
-
     _ripe = {
         IPBaseKeys.NET_RANGE: r"inetnum: *(.+)",
         IPBaseKeys.NET_NAME: r"netname: *(.+)",
