@@ -1,20 +1,16 @@
 import asyncio
+import ipaddress
 import re
 import socket
-import sys
-from typing import Tuple, Generator
-from contextlib import contextmanager
-
-# different installs for async contextmanager based on python version
-if sys.version_info < (3, 7):
-    from async_generator import asynccontextmanager
-else:
-    from contextlib import asynccontextmanager
+from typing import Tuple, Generator, Union
+from contextlib import contextmanager, asynccontextmanager
 
 from python_socks.sync import Proxy
 from python_socks.async_.asyncio import Proxy as AsyncProxy
 
-BLOCKSIZE = 1024
+from .servers import IPv4Allocations, CountryCodeTLD, GenericTLD, SponsoredTLD
+
+BLOCKSIZE = 1500
 
 
 class Query:
@@ -23,35 +19,16 @@ class Query:
     refer_regex = r"refer: *(.+)"
     whois_server_regex = r".+ whois server: *(.+)"
 
-    def __init__(
-        self,
-        server: str = None,
-        authoritative_only: bool = True,
-        proxy_url: str = None,
-        timeout: int = 10,
-    ):
-        self.server = server
-        self.authoritative_only = authoritative_only
+    def __init__(self, proxy_url: str = None, timeout: int = 10):
         self.proxy_url = proxy_url
         self.timeout = timeout
-        self.authoritative_server = ""
-        self.query_output = ""
-        self.query_chain = ""
 
     @staticmethod
     def _find_match(regex: str, blob: str) -> str:
         match = ""
         found = re.search(regex, blob, flags=re.IGNORECASE)
         if found:
-            match = (
-                found.group(1)
-                .rstrip("\r")
-                .replace(" ", "")
-                .replace("http://", "")
-                .replace("https://", "")
-                .rstrip(":")
-                .rstrip("/")
-            )
+            match = found.group(1).rstrip("\r").replace(" ", "").rstrip(":").rstrip("/")
         return match
 
     @contextmanager
@@ -64,7 +41,7 @@ class Query:
             if proxy_url:
                 proxy = Proxy.from_url(proxy_url)
                 # proxy is a standard python socket in blocking mode
-                s = proxy.connect(*address)
+                s = proxy.connect(*address, timeout=self.timeout)
             else:
                 # otherwise use socket
                 s = socket.create_connection(address, self.timeout)
@@ -83,7 +60,7 @@ class Query:
         if proxy_url:
             proxy = AsyncProxy.from_url(proxy_url)
             # sock is a standard python socket in blocking mode
-            sock = await proxy.connect(*address)
+            sock = await proxy.connect(*address, timeout=self.timeout)
             # pass it to asyncio
             s = asyncio.open_connection(host=None, port=None, sock=sock)
         else:
@@ -124,33 +101,64 @@ class Query:
                 result += received.decode("utf-8", errors="ignore")
         return result
 
-    def _do_query(self, server: str, data: str, regex: str) -> str:
+    def run(self, search_term: str, server: str = None) -> list[str]:
+        """
+        Submits the `search_term` to the WHOIS server and returns a list of query responses.
+        """
+        data = search_term + "\r\n"
+        if not server:
+            # TODO: think about moving this to subclass
+            if ":" in data:  # ipv6
+                server_regex = r"whois: *(.+)"
+            else:
+                server_regex = self.refer_regex
+            server = self.iana_server
+        else:
+            server_regex = self.whois_server_regex
+        return self._do_query(server, data, server_regex, [])
+
+    async def aio_run(self, search_term: str, server: str = None) -> list[str]:
+        data = search_term + "\r\n"
+        if not server:
+            if ":" in data:  # ipv6
+                server_regex = r"whois: *(.+)"
+            else:
+                server_regex = self.refer_regex
+            server = self.iana_server
+        else:
+            server_regex = self.whois_server_regex
+        return await self._aio_do_query(server, data, server_regex, [])
+
+    def _do_query(
+        self, server: str, data: str, regex: str, chain: list[str]
+    ) -> list[str]:
         """
         Recursively submits WHOIS queries until it reaches the Authoritative Server.
-        Additionally, if `authoritative_only` is False, all text output from server hops
-        is saved into `query_chain`.
         """
-        # save authoritative server
-        self.authoritative_server = server
         # connect to whois://<server>:43
         with self._create_connection((server, self.whois_port), self.proxy_url) as conn:
             # submit domain and receive raw query output
             query_output = self._send_and_recv(conn, data)
-            if not self.authoritative_only:
-                # concatenate query outputs
-                self.query_chain += query_output
+            # save query chain
+            chain.append(query_output)
             # parse response for the referred WHOIS server name
             whois_server = self._find_match(regex, query_output)
             whois_server = whois_server.lower()
-            if whois_server and whois_server != server:
+            if (
+                whois_server
+                and whois_server != server
+                and not whois_server.startswith("http")
+            ):
                 # recursive call to find more authoritative server
-                query_output = self._do_query(
-                    whois_server, data, self.whois_server_regex
+                chain = self._do_query(
+                    whois_server, data, self.whois_server_regex, chain
                 )
-        # return the WHOIS query output
-        return query_output
+        # return the WHOIS query chain
+        return chain
 
-    async def _aio_do_query(self, server: str, data: str, regex: str):
+    async def _aio_do_query(
+        self, server: str, data: str, regex: str, chain: list[str]
+    ) -> list[str]:
         # connect to whois://<server>:43
         async with self._aio_create_connection(
             (server, self.whois_port), self.proxy_url
@@ -158,118 +166,89 @@ class Query:
             # socket reader and writer
             reader, writer = r_and_w
             # submit domain and receive raw query output
-            query_output = await self._aio_send_and_recv(reader, writer, data)
-            if not self.authoritative_only:
-                # concatenate query outputs
-                self.query_chain += query_output
+            query_output = await asyncio.wait_for(
+                self._aio_send_and_recv(reader, writer, data), self.timeout
+            )
+            chain.append(query_output)
             # parse response for the referred WHOIS server name
             whois_server = self._find_match(regex, query_output)
             whois_server = whois_server.lower()
-            if whois_server and whois_server != server:
+            if (
+                whois_server
+                and whois_server != server
+                and not whois_server.startswith("http")
+            ):
                 # recursive call to find the authoritative server
-                query_output = await self._aio_do_query(
-                    whois_server, data, self.whois_server_regex
+                chain = await self._aio_do_query(
+                    whois_server, data, self.whois_server_regex, chain
                 )
-        # return the WHOIS query output
-        return query_output
+        # return the WHOIS query chain
+        return chain
 
 
 class DomainQuery(Query):
     def __init__(
         self,
-        domain: str,
         server: str = None,
-        authoritative_only: bool = True,
         proxy_url: str = None,
         timeout: int = 10,
     ):
-        super().__init__(server, authoritative_only, proxy_url, timeout)
-        self.domain = domain
+        super().__init__(proxy_url, timeout)
+        self.server = server
 
-    @classmethod
-    def new(
-        cls,
-        domain: str,
-        server: str = None,
-        authoritative_only: bool = True,
-        proxy_url: str = None,
-        timeout: int = 10,
-    ):
-        _self = cls(domain, server, authoritative_only, proxy_url, timeout)
-        data = domain + "\r\n"
-        if not _self.server:
-            server_regex = _self.refer_regex
-            server = _self.iana_server
-        else:
-            server_regex = _self.whois_server_regex
-            server = _self.server
-        _self.query_output = _self._do_query(server, data, server_regex)
-        return _self
+    @staticmethod
+    def _get_server_name(tld: str) -> Union[str, None]:
+        tld_converted = tld.upper().replace("-", "_")
+        for servers in [CountryCodeTLD, GenericTLD, SponsoredTLD]:
+            if hasattr(servers, tld_converted):
+                server = getattr(servers, tld_converted)
+                return server
+        return None
 
-    @classmethod
-    async def new_aio(
-        cls,
-        domain: str,
-        server: str = None,
-        authoritative_only: bool = True,
-        proxy_url: str = None,
-        timeout: int = 10,
-    ):
-        _self = cls(domain, server, authoritative_only, proxy_url, timeout)
-        data = domain + "\r\n"
-        if not _self.server:
-            server_regex = _self.refer_regex
-            server = _self.iana_server
-        else:
-            server_regex = _self.whois_server_regex
-            server = _self.server
-        _self.query_output = await _self._aio_do_query(server, data, server_regex)
-        return _self
+    def run(self, search_term: str, server: str = None) -> list[str]:
+        if not server:
+            server = self._get_server_name(search_term)
+        return super().run(str(search_term), server)
+
+    async def aio_run(self, search_term: str, server: str = None) -> list[str]:
+        if not server:
+            server = self._get_server_name(search_term)
+        return await super().aio_run(str(search_term), server)
 
 
 class NumberQuery(Query):
     def __init__(
         self,
-        ip: str,  # ipv4 or ipv6
         server: str = None,
-        authoritative_only: bool = True,
         proxy_url: str = None,
         timeout: int = 10,
     ):
-        super().__init__(server, authoritative_only, proxy_url, timeout)
+        super().__init__(proxy_url, timeout)
+        self.server = server
         self.whois_server_regex = r"ReferralServer: *whois://(.+)"
-        self.ip = ip
 
-    @classmethod
-    def new(
-        cls,
-        ip: str,  # ipv4 or ipv6; validation is handled by caller.
-        server: str = None,
-        authoritative_only: bool = False,
-        proxy_url: str = None,
-        timeout: int = 10,
-    ):
-        _self = cls(ip, server, authoritative_only, proxy_url, timeout)
-        data = ip + "\r\n"
-        if ":" in ip:  # ipv6
-            _self.refer_regex = r"whois: *(.+)"
-        server = server or _self.iana_server
-        _self.query_output = _self._do_query(server, data, _self.refer_regex)
-        return _self
+    @staticmethod
+    def _get_server_name(ip: Union[ipaddress.IPv4Address, ipaddress.IPv6Address]):
+        if isinstance(ip, ipaddress.IPv4Address):
+            _, server = IPv4Allocations().get_servers(ip)
+            return server
+        elif isinstance(ip, ipaddress.IPv6Address):
+            return None
 
-    @classmethod
-    async def new_aio(
-        cls,
-        ip: str,
+    def run(
+        self,
+        search_term: Union[ipaddress.IPv4Address, ipaddress.IPv6Address],
         server: str = None,
-        authoritative_only: bool = False,
-        proxy_url: str = None,
-        timeout: int = 10,
-    ):
-        _self = cls(ip, server, authoritative_only, proxy_url, timeout)
-        data = ip + "\r\n"
-        if ":" in ip:  # ipv6
-            _self.refer_regex = r"whois: *(.+)"
-        server = server or _self.iana_server
-        _self.query_output = await _self._aio_do_query(server, data, _self.refer_regex)
-        return _self
+    ) -> list[str]:
+        if not server:
+            server = self._get_server_name(search_term)
+        return super().run(str(search_term), server)
+
+    async def aio_run(
+        self,
+        search_term: Union[ipaddress.IPv4Address, ipaddress.IPv6Address],
+        server: str = None,
+    ) -> list[str]:
+        if not server:
+            server = self._get_server_name(search_term)
+        return await super().aio_run(str(search_term), server)
